@@ -1,70 +1,53 @@
 ## define the pmcmc class
 setClass(
          'pmcmc',
+         contains='pfilterd.pomp',
          representation(
                         pars = 'character',
                         Nmcmc = 'integer',
                         dprior = 'function',
                         hyperparams = 'list',
-                        Np = 'integer',
                         random.walk.sd = 'numeric',
-                        filter.mean = 'matrix',
                         conv.rec = 'matrix',
-                        eff.sample.size = 'numeric',
-                        cond.loglik = 'numeric',
-                        loglik = 'numeric',
                         log.prior = 'numeric'
-                        ),
-         contains='pomp'
+                        )
          )
 
 ## PMCMC algorithm functions
-pmcmc <- function (object, ... )
-  stop("function ",sQuote("pmcmc")," is undefined for objects of class ",sQuote(class(object)))
-setGeneric('pmcmc')
+setGeneric('pmcmc',function(object,...)standardGeneric("pmcmc"))
 
-dprior <- function (object, params, log)
-  stop("function ",sQuote("dprior")," is undefined for objects of class ",sQuote(class(object)))
-setGeneric('dprior')
+setGeneric('dprior', function(object,...)standardGeneric("dprior"))
 
 setMethod(
           "dprior",
-          "pmcmc",
-          function (object, params, log = FALSE) {
+          signature=signature(object="pmcmc"),
+          function (object, params, log = FALSE, ...) {
             do.call(object@dprior,list(params=params,hyperparams=object@hyperparams,log=log))
           }
           )
 
-pmcmc.internal <- function (object, Nmcmc = 1,
-                            start = NULL,
-                            pars = NULL, 
-                            dprior.fun = NULL,
-                            rw.sd = NULL,
-                            Np = NULL,
-                            hyperparams = NULL,
-                            tol = 1e-17, max.fail = 0,
-                            verbose = FALSE,
-                            .ndone = 0, .prevcomp = NULL
-                            ) {
-  is.pmcmc <- is(object,"pmcmc")
-  if (is.null(start)) {
-    start <- coef(object)
-    if (length(start)==0)
-      stop("pmcmc error: ",sQuote("start")," must be specified if ",
-           sQuote("coef(object)")," is NULL",
-           call.=FALSE)
-  } else if (length(start)==0)
-    stop("pmcmc error: ",sQuote("start")," must be a named vector",call.=FALSE)
+pmcmc.internal <- function (object, Nmcmc,
+                            start, pars, dprior.fun,
+                            rw.sd, Np,
+                            hyperparams,
+                            tol, max.fail,
+                            verbose,
+                            .ndone, .prevcomp) {
+
+  if (missing(start))
+    stop("pmcmc error: ",sQuote("start")," must be specified",call.=FALSE)
+  if (length(start)==0)
+    stop(
+         "pmcmc error: ",sQuote("start")," must be specified if ",
+         sQuote("coef(object)")," is NULL",
+         call.=FALSE
+         )
   start.names <- names(start)
   if (is.null(start.names))
     stop("pmcmc error: ",sQuote("start")," must be a named vector",call.=FALSE)
 
-  if (is.null(rw.sd)) {
-    if (is.pmcmc)
-      rw.sd <- object@random.walk.sd
-    else
-      stop("pmcmc error: ",sQuote("rw.sd")," must be specified",call.=FALSE)
-  }
+  if (missing(rw.sd))
+    stop("pmcmc error: ",sQuote("rw.sd")," must be specified",call.=FALSE)
   rw.names <- names(rw.sd)
   if (is.null(rw.names) || any(rw.sd<0))
     stop("pmcmc error: ",sQuote("rw.sd")," must be a named non-negative numerical vector",call.=FALSE)
@@ -74,12 +57,8 @@ pmcmc.internal <- function (object, Nmcmc = 1,
   if (length(rw.names) == 0)
     stop("pmcmc error: ",sQuote("rw.sd")," must have one positive entry for each parameter to be estimated",call.=FALSE)
 
-  if (is.null(pars)) {
-    if (is.pmcmc) 
-      pars <- object@pars
-    else
-      stop("pmcmc error: ",sQuote("pars")," must be specified",call.=FALSE)
-  }
+  if (missing(pars))
+    stop("pmcmc error: ",sQuote("pars")," must be specified",call.=FALSE)
   if (length(pars)==0)
     stop("pmcmc error: at least one parameter must be estimated",call.=FALSE)
   if (
@@ -111,30 +90,20 @@ pmcmc.internal <- function (object, Nmcmc = 1,
   rw.sd <- rw.sd[pars]
   rw.names <- names(rw.sd)
 
-  if (is.null(dprior.fun)) {
-    if (is.pmcmc) 
-      dprior.fun <- object@dprior
-    else
-      stop("pmcmc error: ",sQuote("dprior")," must be specified",call.=FALSE)
-  }
+  if (missing(dprior.fun))
+    stop("pmcmc error: ",sQuote("dprior")," must be specified",call.=FALSE)
 
-  if (is.null(Np)) {
-    if (is.pmcmc) 
-      Np <- object@Np
-    else
-      stop("pmcmc error: ",sQuote("Np")," must be specified",call.=FALSE)
-  }
+  if (missing(Np))
+    stop("pmcmc error: ",sQuote("Np")," must be specified",call.=FALSE)
   Np <- as.integer(Np)
   if ((length(Np)!=1)||(Np < 1))
     stop("pmcmc error: ",sQuote("Np")," must be a positive integer",call.=FALSE)
 
-  if (is.null(hyperparams)) {
-    if (is.pmcmc) 
-      hyperparams <- object@hyperparams
-    else
-      stop("pmcmc error: ",sQuote("hyperparams")," must be specified",call.=FALSE)
-  }
+  if (missing(hyperparams))
+    stop("pmcmc error: ",sQuote("hyperparams")," must be specified",call.=FALSE)
 
+  if (missing(Nmcmc))
+    stop("pmcmc error: ",sQuote("Nmcmc")," must be specified",call.=FALSE)
   Nmcmc <- as.integer(Nmcmc)
   if (Nmcmc<0)
     stop("pmcmc error: ",sQuote("Nmcmc")," must be a positive integer",call.=FALSE)
@@ -171,41 +140,38 @@ pmcmc.internal <- function (object, Nmcmc = 1,
          )
   }
 
-  obj <- new(
-             "pmcmc",
-             as(object,"pomp"),
-             pars=pars,
-             Nmcmc=0L,
-             dprior=dprior.fun,
-             Np=Np,
-             hyperparams=hyperparams,
-             random.walk.sd=rw.sd
-             )
+  obj <- as(object,"pomp")
+  
+  if (Nmcmc>0)
+    tmp.pmcmc <- new("pmcmc",obj,dprior=dprior.fun,hyperparams=hyperparams)
+  else
+    pfp <- obj
 
   if (.ndone==0) { ## compute prior and likelihood on initial parameter vector
-    x <- try(
-             pfilter.internal(
-                              object=obj,
-                              params=theta,
-                              Np=Np,
-                              tol=tol,
-                              max.fail=max.fail,
-                              pred.mean=FALSE,
-                              pred.var=FALSE,
-                              filter.mean=TRUE,
-                              save.states=FALSE,
-                              verbose=verbose
-                              ),
-             silent=FALSE
-             )
-    if (inherits(x,'try-error'))
+    pfp <- try(
+               pfilter.internal(
+                                object=obj,
+                                params=theta,
+                                Np=Np,
+                                tol=tol,
+                                max.fail=max.fail,
+                                pred.mean=FALSE,
+                                pred.var=FALSE,
+                                filter.mean=TRUE,
+                                save.states=FALSE,
+                                verbose=verbose
+                                ),
+               silent=FALSE
+               )
+    if (inherits(pfp,'try-error'))
       stop("pmcmc error: error in ",sQuote("pfilter"),call.=FALSE)
-    x$log.prior <- dprior(object=obj,params=theta,log=TRUE)
+    log.prior <- dprior(object=tmp.pmcmc,params=theta,log=TRUE)
   } else { ## has been computed previously
-    x <- .prevcomp
+    pfp <- .prevcomp$pfp
+    log.prior <- .prevcomp$log.prior
   }
   conv.rec[1,names(theta)] <- theta
-  conv.rec[1,c(1,2,3)] <- c(x$loglik,x$log.prior,x$nfail)
+  conv.rec[1,c(1,2,3)] <- c(pfp@loglik,log.prior,pfp@nfail)
 
   for (n in seq_len(Nmcmc)) { # main loop
 
@@ -213,74 +179,79 @@ pmcmc.internal <- function (object, Nmcmc = 1,
     theta.prop[pars] <- rnorm(n=length(pars),mean=theta.prop[pars],sd=rw.sd)
 
     ## run the particle filter on the proposed new parameter values
-    x.prop <- try(
-                  pfilter.internal(
-                                   object=obj,
-                                   params=theta.prop,
-                                   Np=Np,
-                                   tol=tol,
-                                   max.fail=max.fail,
-                                   pred.mean=FALSE,
-                                   pred.var=FALSE,
-                                   filter.mean=TRUE,
-                                   save.states=FALSE,
-                                   verbose=verbose
-                                   ),
-                  silent=FALSE
-                  )
-    if (inherits(x.prop,'try-error'))
+    pfp.prop <- try(
+                    pfilter.internal(
+                                     object=obj,
+                                     params=theta.prop,
+                                     Np=Np,
+                                     tol=tol,
+                                     max.fail=max.fail,
+                                     pred.mean=FALSE,
+                                     pred.var=FALSE,
+                                     filter.mean=TRUE,
+                                     save.states=FALSE,
+                                     verbose=verbose
+                                     ),
+                    silent=FALSE
+                    )
+    if (inherits(pfp.prop,'try-error'))
       stop("pmcmc error: error in ",sQuote("pfilter"),call.=FALSE)
-    x.prop$log.prior <- dprior(object=obj,params=theta.prop,log=TRUE)
+    log.prior.prop <- dprior(object=tmp.pmcmc,params=theta.prop,log=TRUE)
 
     ## PMCMC update rule (OK because proposal is symmetric)
-    if (runif(1) < exp(x.prop$loglik-x$loglik+x.prop$log.prior-x$log.prior)) {
+    if (runif(1) < exp(pfp.prop@loglik-pfp@loglik+log.prior-log.prior.prop)) {
+      pfp <- pfp.prop
       theta <- theta.prop
-      x <- x.prop
     }
 
     ## store a record of this iteration
     conv.rec[n+1,names(theta)] <- theta
-    conv.rec[n+1,c(1,2,3)] <- c(x$loglik,x$log.prior,x$nfail)
+    conv.rec[n+1,c(1,2,3)] <- c(pfp@loglik,log.prior,pfp@nfail)
 
     if (verbose) cat("PMCMC iteration ",n," of ",Nmcmc," completed\n")
 
   }
 
-  coef(obj) <- theta
-
-  if (Nmcmc>0) {
-    obj@Nmcmc <- as.integer(Nmcmc)
-    obj@filter.mean <- x$filter.mean
-    obj@conv.rec <- conv.rec
-    obj@eff.sample.size <- x$eff.sample.size
-    obj@cond.loglik <- x$cond.loglik
-    obj@loglik <- x$loglik
-    obj@log.prior <- x$log.prior
-  }
-
-  obj
+  new(
+      "pmcmc",
+      pfp,
+      params=theta,
+      Nmcmc=Nmcmc,
+      pars=pars,
+      dprior=dprior.fun,
+      random.walk.sd=rw.sd,
+      Np=Np,
+      hyperparams=hyperparams,
+      tol=tol,
+      conv.rec=conv.rec,
+      log.prior=log.prior
+      )
 }
 
 setMethod(
           "pmcmc",
-          "pomp",
-          function (
-                    object, Nmcmc = 1,
+          signature=signature(object="pomp"),
+          function (object, Nmcmc = 1,
                     start, pars, rw.sd,
                     dprior, Np, hyperparams,
-                    tol = 1e-17, max.fail = 0, verbose = getOption("verbose")
-                    )
-          {
-            if (missing(start)) start <- NULL
+                    tol = 1e-17, max.fail = 0, verbose = getOption("verbose"),
+                    ...) {
+            
+            if (missing(start)) start <- coef(object)
+
             if (missing(rw.sd))
               stop("pmcmc error: ",sQuote("rw.sd")," must be specified",call.=FALSE)
+
             if (missing(pars)) {
               pars <- names(rw.sd)[rw.sd>0]
             }
+
             if (missing(Np))
               stop("pmcmc error: ",sQuote("Np")," must be specified",call.=FALSE)
+
             if (missing(hyperparams))
               stop("pmcmc error: ",sQuote("hyperparams")," must be specified",call.=FALSE)
+
             if (missing(dprior)) {         # use default flat improper prior
               dprior <- function (params, hyperparams, log) {
                 if (log) 0 else 1
@@ -309,41 +280,145 @@ setMethod(
                            tol=tol,
                            max.fail=max.fail,
                            verbose=verbose,
-                           .ndone=0
+                           .ndone=0,
+                           .prevcomp=NULL
                            )
           }
           )
-          
 
 setMethod(
           "pmcmc",
+          signature=signature(object="pfilterd.pomp"),
+          function (object, Nmcmc = 1,
+                    start, pars, rw.sd,
+                    dprior, Np, hyperparams,
+                    tol, max.fail = 0, verbose = getOption("verbose"),
+                    ...) {
+            
+            if (missing(start)) start <- coef(object)
+
+            if (missing(rw.sd))
+              stop("pmcmc error: ",sQuote("rw.sd")," must be specified",call.=FALSE)
+
+            if (missing(pars)) {
+              pars <- names(rw.sd)[rw.sd>0]
+            }
+
+            if (missing(Np)) Np <- object@Np
+
+            if (missing(hyperparams))
+              stop("pmcmc error: ",sQuote("hyperparams")," must be specified",call.=FALSE)
+
+            if (missing(dprior)) {         # use default flat improper prior
+              dprior <- function (params, hyperparams, log) {
+                if (log) 0 else 1
+              }
+            } else {
+              dprior <- match.fun(dprior)
+              if (!all(c('params','hyperparams','log')%in%names(formals(dprior))))
+                stop(
+                     "pmcmc error: ",
+                     sQuote("dprior"),
+                     " must be a function of prototype ",
+                     sQuote("dprior(params,hyperparams,log)"),
+                     call.=FALSE
+                     )
+            }
+              
+            if (missing(tol)) tol <- object@tol
+            
+            pmcmc.internal(
+                           object=as(object,"pomp"),
+                           Nmcmc=Nmcmc,
+                           start=start,
+                           pars=pars,
+                           dprior.fun=dprior,
+                           rw.sd=rw.sd,
+                           Np=Np,
+                           hyperparams=hyperparams,
+                           tol=tol,
+                           max.fail=max.fail,
+                           verbose=verbose,
+                           .ndone=0,
+                           .prevcomp=NULL
+                           )
+          }
+          )
+
+setMethod(
           "pmcmc",
-          function (object, Nmcmc, ...)
-          {
+          signature=signature(object="pmcmc"),
+          function (object, Nmcmc,
+                    start, pars, rw.sd,
+                    dprior, Np, hyperparams,
+                    tol, max.fail = 0, verbose = getOption("verbose"),
+                    ...) {
+
             if (missing(Nmcmc)) Nmcmc <- object@Nmcmc
-            pmcmc.internal(object,Nmcmc=Nmcmc,...)
+            if (missing(start)) start <- coef(object)
+            if (missing(pars)) pars <- object@pars
+            if (missing(rw.sd)) pars <- object@random.walk.sd
+            if (missing(dprior)) dprior <- object@dprior
+            if (missing(Np)) Np <- object@Np
+            if (missing(hyperparams)) hyperparams <- object@hyperparams
+            if (missing(tol)) tol <- object@tol
+
+            pmcmc.internal(
+                           object=as(object,"pomp"),
+                           Nmcmc=Nmcmc,
+                           start=start,
+                           pars=pars,
+                           dprior.fun=dprior,
+                           rw.sd=rw.sd,
+                           Np=Np,
+                           hyperparams=hyperparams,
+                           tol=tol,
+                           max.fail=max.fail,
+                           verbose=verbose,
+                           .ndone=0,
+                           .prevcomp=NULL
+                           )
           }
           )
 
 setMethod(
           'continue',
-          'pmcmc',
-          function (object, Nmcmc = 1, ...) {
+          signature=signature(object='pmcmc'),
+          function (object, Nmcmc = 1,
+                    start, pars, rw.sd,
+                    dprior, Np, hyperparams,
+                    tol, max.fail = 0, verbose = getOption("verbose"),
+                    ...) {
+
+            if (missing(start)) start <- coef(object)
+            if (missing(pars)) pars <- object@pars
+            if (missing(rw.sd)) rw.sd <- object@random.walk.sd
+            if (missing(dprior)) dprior <- object@dprior
+            if (missing(Np)) Np <- object@Np
+            if (missing(hyperparams)) hyperparams <- object@hyperparams
+            if (missing(tol)) tol <- object@tol
+
             ndone <- object@Nmcmc
+
             obj <- pmcmc.internal(
-                                  object=object,
+                                  object=as(object,"pomp"),
                                   Nmcmc=Nmcmc,
+                                  start=start,
+                                  pars=pars,
+                                  dprior.fun=dprior,
+                                  rw.sd=rw.sd,
+                                  Np=Np,
+                                  hyperparams=hyperparams,
+                                  tol=tol,
+                                  max.fail=max.fail,
+                                  verbose=verbose,
                                   .ndone=ndone,
                                   .prevcomp=list(
-                                    log.prior=object@log.prior,
-                                    loglik=object@loglik,
-                                    nfail=object@conv.rec[ndone+1,"nfail"],
-                                    filter.mean=object@filter.mean,
-                                    eff.sample.size=object@eff.sample.size,
-                                    cond.loglik=object@cond.loglik
-                                    ),
-                                  ...
+                                    pfp=as(object,"pfilterd.pomp"),
+                                    log.prior=object@log.prior
+                                    )
                                   )
+
             obj@conv.rec <- rbind(
                                   object@conv.rec[,colnames(obj@conv.rec)],
                                   obj@conv.rec[-1,]
